@@ -37,24 +37,15 @@ from hashlib import sha1
 import logging
 import multiprocessing
 import os
+import socket
 import sys
 import tempfile
 import threading
 import urllib
 
-# List of third-party libraries that we'll insert into sys.path.
-THIRD_PARTY_LIBS = [
-    'socksipy-branch',
-    'retry-decorator',
-    'google-api-python-client',
-    'boto',
-]
-for libdir in THIRD_PARTY_LIBS:
-  sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)) +
-      '/../third_party/%s' % libdir)
-
 from boto import config
 import httplib2
+from oauth2client.anyjson import simplejson
 from oauth2client.client import AccessTokenRefreshError
 from oauth2client.client import HAS_CRYPTO
 from oauth2client.client import OAuth2Credentials
@@ -78,6 +69,15 @@ except:
 
 GSUTIL_DEFAULT_SCOPE = 'https://www.googleapis.com/auth/devstorage.full_control'
 
+META_TOKEN_URI = ('http://metadata/computeMetadata/v1/instance/'
+                  'service-accounts/default/token')
+
+META_HEADERS = {
+    'X-Google-Metadata-Request': 'True'
+}
+
+METADATA_SERVER = 'http://169.254.169.254'
+
 
 # Note: this is copied from gsutil's gslib.cred_types. It should be kept in
 # sync. Also note that this library does not use HMAC, but it's preserved from
@@ -398,7 +398,7 @@ class OAuth2ServiceAccountClient(OAuth2Client):
 
 
 class GsAccessTokenRefreshError(Exception):
-  """Rate limiting error when exchanging refresh token for access token."""
+  """Transient error when requesting access token."""
   def __init__(self, e):
     super(Exception, self).__init__(e)
 
@@ -491,6 +491,62 @@ cases in which you will see this error are:
         raise
 
 
+class OAuth2GCEClient(OAuth2Client):
+  """OAuth2 client for GCE instance."""
+
+  def __init__(self):
+    super(OAuth2GCEClient, self).__init__(
+        cache_key_base='',
+        # Only InMemoryTokenCache can be used with empty cache_key_base.
+        access_token_cache=InMemoryTokenCache())
+
+  @Retry(GsAccessTokenRefreshError,
+         tries=6,
+         timeout_secs=1)
+  def FetchAccessToken(self):
+    response = None
+    try:
+      http = httplib2.Http()
+      response, content = http.request(META_TOKEN_URI, method='GET',
+                                       body=None, headers=META_HEADERS)
+    except Exception:
+      raise GsAccessTokenRefreshError()
+
+    if response.status == 200:
+      d = simplejson.loads(content)
+
+      return AccessToken(
+          d['access_token'],
+          datetime.datetime.now() +
+              datetime.timedelta(seconds=d.get('expires_in', 0)),
+          datetime_strategy=self.datetime_strategy)
+
+
+def _IsGCE():
+  try:
+    http = httplib2.Http()
+    response, _ = http.request(METADATA_SERVER)
+    return response.status == 200
+
+  except (httplib2.ServerNotFoundError, socket.error):
+    # We might see something like "No route to host" propagated as a socket
+    # error. We might also catch transient socket errors, but at that point
+    # we're going to fail anyway, just with a different error message. With
+    # this approach, we'll avoid having to enumerate all possible non-transient
+    # socket errors.
+    return False
+  except Exception, e:
+    LOG.warning("Failed to determine whether we're running on GCE, so we'll"
+                "assume that we aren't: %s", e)
+    return False
+
+  return False
+
+
+def CreateOAuth2GCEClient():
+  return OAuth2GCEClient() if _IsGCE() else None
+
+
 class AccessToken(object):
   """Encapsulates an OAuth2 access token."""
 
